@@ -1,6 +1,6 @@
 /* gcompris - shapegame.c
  *
- * Time-stamp: <2001/05/24 23:29:57 bruno>
+ * Time-stamp: <2001/06/26 23:48:39 bruno>
  *
  * Copyright (C) 2000 Bruno Coudoin
  *
@@ -29,6 +29,8 @@
 
 #define SOUNDLISTFILE PACKAGE
 
+#define UNDEFINED "Undefined"
+
 static GList *item_list = NULL;
 
 static GcomprisBoard *gcomprisBoard = NULL;
@@ -46,6 +48,8 @@ typedef struct _Shape Shape;
 struct _Shape {
   char  *name;			/* name of the shape */
   char  *pixmapfile;		/* relative pixmap file name of the shape */
+  char  *targetfile;		/* OPTIONAL relative pixmap file name of the target shape, by default 
+				   a red point is displayed */
   double x;			/* x coordinate */
   double y;			/* y coordinate */
   double w;			/* width */
@@ -85,7 +89,7 @@ static void shapegame_next_level(void);
 static gboolean read_xml_file(char *fname);
 static gboolean write_xml_file(char *fname);
 static Shape *find_closest_shape(double x, double y, double limit);
-static Shape *create_shape(ShapeType type, char *name, char *pixmapfile, 
+static Shape *create_shape(ShapeType type, char *name, char *pixmapfile,  char *targetfile,
 			   double x, double y, double l, double h, double zoomx, 
 			   double zoomy, guint position);
 static void process_ok();
@@ -466,7 +470,7 @@ add_shape_to_list_of_shapes(Shape *shape)
 					NULL);
 	  gdk_pixbuf_unref(pixmap);
 
-	  icon_shape = create_shape(SHAPE_ICON, shape->name, shape->pixmapfile, 
+	  icon_shape = create_shape(SHAPE_ICON, shape->name, shape->pixmapfile, shape->targetfile, 
 				    (double)0, (double)y_offset,
 				    (double)w, (double)h, 
 				    (double)1, (double)1,
@@ -653,6 +657,8 @@ item_event(GnomeCanvasItem *item, GdkEvent *event, Shape *shape)
 	       gnome_canvas_item_move(item, 
 				      targetshape->x - x + offset_x, 
 				      targetshape->y - y + offset_y);
+	       /* Hide it */
+	       gnome_canvas_item_hide(item);
 
 	       if(strcmp(targetshape->name, shape->name)==0)
 		 {
@@ -703,6 +709,7 @@ item_event(GnomeCanvasItem *item, GdkEvent *event, Shape *shape)
 
 	       /* Set the icon item */
 	       targetshape->icon_shape=shape;
+
 	     }
 	   else
 	     {
@@ -755,6 +762,7 @@ static void
 add_shape_to_canvas(Shape *shape)
 {
   GdkPixbuf *pixmap;
+  GdkPixbuf *targetpixmap;
   GnomeCanvasItem *item;
 
   /* checking of preconditions, this is usefull for tracking down bugs,
@@ -764,19 +772,37 @@ add_shape_to_canvas(Shape *shape)
   pixmap = gcompris_load_pixmap(shape->pixmapfile);
   if(pixmap)
     {	
-      /* Display a point to display the target location of this shape */
       if(shape->type==SHAPE_TARGET)
 	{
-	  item = gnome_canvas_item_new (GNOME_CANVAS_GROUP(shape_root_item),
-					gnome_canvas_ellipse_get_type(),
-					"x1", (double)shape->x-2.5,
-					"y1", (double)shape->y-2.5,
-					"x2", (double)shape->x+2.5,
-					"y2", (double)shape->y+2.5,
-					"fill_color_rgba", 0xEf000080,
-					"outline_color", "black",
-					"width_pixels", 0,
-					NULL);
+	  if(strcmp(shape->targetfile, UNDEFINED)!=0)
+	    {
+	      targetpixmap = gcompris_load_pixmap(shape->targetfile);
+	      item = gnome_canvas_item_new (GNOME_CANVAS_GROUP(shape_root_item),
+					    gnome_canvas_pixbuf_get_type (),
+					    "pixbuf", targetpixmap, 
+					    "x", (double)shape->x-(gdk_pixbuf_get_width(targetpixmap) * shape->zoomx)/2,
+					    "y", (double)shape->y-(gdk_pixbuf_get_height(targetpixmap) * shape->zoomy)/2,
+					    "width", (double) gdk_pixbuf_get_width(targetpixmap) * shape->zoomx,
+					    "height", (double) gdk_pixbuf_get_height(targetpixmap) * shape->zoomy,
+					    "width_set", TRUE, 
+					    "height_set", TRUE,
+					    NULL);
+	      gdk_pixbuf_unref(targetpixmap);
+	    } 
+	  else
+	    {
+	      /* Display a point to highlight the target location of this shape */
+	      item = gnome_canvas_item_new (GNOME_CANVAS_GROUP(shape_root_item),
+					    gnome_canvas_ellipse_get_type(),
+					    "x1", (double)shape->x-2.5,
+					    "y1", (double)shape->y-2.5,
+					    "x2", (double)shape->x+2.5,
+					    "y2", (double)shape->y+2.5,
+					    "fill_color_rgba", 0xEf000080,
+					    "outline_color", "black",
+					    "width_pixels", 0,
+					    NULL);
+	    }
 	  gnome_canvas_item_lower_to_bottom(item);
 	  item_list = g_list_append (item_list, item);
 	}
@@ -838,7 +864,7 @@ void create_title(char *name, double x, double y, char *justification)
 }
 
 static Shape *
-create_shape(ShapeType type, char *name, char *pixmapfile, double x, double y, 
+create_shape(ShapeType type, char *name, char *pixmapfile, char *targetfile, double x, double y, 
 	     double w, double h, double zoomx, 
 	     double zoomy, guint position)
 {
@@ -849,6 +875,7 @@ create_shape(ShapeType type, char *name, char *pixmapfile, double x, double y,
   
   shape->name = g_strdup(name);
   shape->pixmapfile = g_strdup(pixmapfile);
+  shape->targetfile = g_strdup(targetfile);
   shape->x = x;
   shape->y = y;
   shape->w = w;
@@ -873,7 +900,7 @@ create_shape(ShapeType type, char *name, char *pixmapfile, double x, double y,
 static void
 add_xml_shape_to_data(xmlNodePtr xmlnode, GNode * child)
 {
-  char *name, *pixmapfile, *cx, *cy, *czoomx, *czoomy, *cposition, *ctype, *justification;
+  char *name, *pixmapfile, *targetfile, *cx, *cy, *czoomx, *czoomy, *cposition, *ctype, *justification;
   double x, y, zoomx, zoomy;
   guint position;
   ShapeType type = SHAPE_TARGET;
@@ -892,8 +919,12 @@ add_xml_shape_to_data(xmlNodePtr xmlnode, GNode * child)
   name = xmlGetProp(xmlnode,"name");
 
   pixmapfile = xmlGetProp(xmlnode,"pixmapfile");
-  /* if unspecified, make it "Unknown" */
-  if(!pixmapfile) pixmapfile = "Unknown";
+  /* if unspecified, make it UNDEFINED */
+  if(!pixmapfile) pixmapfile = UNDEFINED;
+
+  targetfile = xmlGetProp(xmlnode,"targetfile");
+  /* if unspecified, make it UNDEFINED */
+  if(!targetfile) targetfile = UNDEFINED;
 
   /* get the X coord of the shape */
   cx = xmlGetProp(xmlnode,"x");
@@ -939,7 +970,7 @@ add_xml_shape_to_data(xmlNodePtr xmlnode, GNode * child)
     {
       /* add the shape to the database */
       /* WARNING : I do not initialize the width and height since I don't need them */
-      shape = create_shape(type, name, pixmapfile, x, y, 
+      shape = create_shape(type, name, pixmapfile, targetfile, x, y, 
 			   (double)0, (double)0,
 			   zoomx, zoomy, position);
       add_shape_to_canvas(shape);
